#!/usr/bin/env python
"""
Create a polar two center grid on which the Slater-Koster integrals are
calculated. Since generating the grid takes a long time we store and reuse it.
This file is a copy of SlaterKoster/generate_ptcgrid.py, but it differs in the 
size of the grid. Since continuum orbitals do not decay at large distances, the
grid has to be much larger, at least the size of the largest interatomic distance.
"""
import numpy
from numpy import linspace, exp, log, pi
import os.path
import pprint
import sys
from DFTB.SlaterKoster.PolarTwoCenterGrid import ptcgrid

# grid for integration
rmin, rmax, Nr = 0.000000001, 100.0, 5000
# number of angular grid points
Na = 150
# interatomic distance
dmin, dmax, Nd = 0.0, 40.0, 500
#
angles = linspace(0.0, pi, Na)
r = linspace(max(1.0e-10, rmin), rmax, Nr)
#r = exp(linspace(log(rmin), log(rmax), Nr))
d = linspace(dmin, dmax, Nd)

##################### DO NOT MESS WITH THE CODE BELOW ################################

# create polar two center grid
grid = ptcgrid(d/2.0,r,angles)

script_dir = os.path.dirname(os.path.realpath(__file__))
slako_dir = os.path.join(script_dir, "slako_tables_scattering/")

fh = open(os.path.join(slako_dir, "double_polar_grid.py"), "w")
numpy.set_printoptions(threshold=sys.maxint)
pp = pprint.PrettyPrinter(depth=10)
print>>fh, "from numpy import load, array"
print>>fh, "import os.path"
print>>fh, "# This file was automatically generated by %s" % sys.argv[0]
print>>fh, "# It loads the double polar grid from a binary file grid.npz that can be used"
print>>fh, "# for integrating overlaps and hamiltonian matrix elements."
print>>fh, "# separation between centers"
print>>fh, "d = \\\n%s" % pp.pformat(d)

# write grid to separate .npz file
grids = [Xk for Xk in grid[0]] + [Yk for Yk in grid[1]] + [area_k for area_k in grid[2]]
grid_file = os.path.join(slako_dir, "grid.npz")
grfh = open(grid_file, "wb")
numpy.savez(grfh, *grids)
grfh.close()

print>>fh, "# load grid from grid.npz"  
print>>fh, "_npz_file = os.path.join(os.path.dirname(os.path.realpath(__file__)), 'grid.npz')"
print>>fh, "_npz = load(open(_npz_file, 'rb'))"
print>>fh, "_gr = [_npz[\"arr_%i\" % i] for i in range(0, len(_npz.keys()))]"
print>>fh, "# grid"
print>>fh, "# (X,Y,areas) = grid[0][k], grid[1][k], grid[2][k]   gives the x and y-positions"
print>>fh, "# of the centers and areas of the quads for a grid with distance d[k] between the centers."
print>>fh, "grid = _gr[:len(d)], _gr[len(d):2*len(d)], _gr[2*len(d):3*len(d)]"

fh.close()

